<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	
  <xsl:include href="assets/resources/xsl/joinedTable.xsl" />
  <xsl:include href="assets/resources/xsl/customFields.xsl" />
  <xsl:include href="assets/resources/xsl/dataSyncServices.xsl" />
  <xsl:include href="assets/resources/xsl/sites.xsl" />
  <xsl:include href="assets/resources/xsl/users.xsl" />
  <xsl:include href="assets/resources/xsl/devices.xsl" />
  <xsl:include href="assets/resources/xsl/groupsAndTeams.xsl" />
  <xsl:include href="assets/resources/xsl/teamMemberships.xsl" />
  <xsl:include href="assets/resources/xsl/customAttributes.xsl" />
  <xsl:include href="assets/resources/xsl/customValues.xsl" />
  <xsl:include href="assets/resources/xsl/eventDomainPredicates.xsl" />
  <xsl:include href="assets/resources/xsl/groupRecurringCoverages.xsl" />
  <xsl:include href="assets/resources/xsl/groupHolidayCoverages.xsl" />
  <xsl:include href="assets/resources/xsl/deviceTimeframe.xsl" />

	<xsl:output method="xml" encoding="UTF-8"
		doctype-system="http://www.springframework.org/dtd/spring-beans.dtd"
		doctype-public="-//SPRING//DTD BEAN//EN"
		cdata-section-elements="beans" />
	<!-- for case insensitive string comparison -->
	<xsl:variable name="uc">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>
	<xsl:variable name="lc">abcdefghijklmnopqrstuvwxyz</xsl:variable>

  <!-- Classes -->
  <xsl:variable name="collection_fieldsync_class">com.invoqsystems.apex.datasynchronizer.processor.field.EnterpriseCollectionFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_tsync_class">com.invoqsystems.apex.datasynchronizer.processor.EnterpriseTableSynchronizerImpl</xsl:variable>
  <xsl:variable name="staging_tsync_class">com.invoqsystems.apex.datasynchronizer.processor.StagingTableSynchronizerImpl</xsl:variable>
	<xsl:variable name="staging_rsync_interface">com.invoqsystems.apex.datasynchronizer.processor.StagingRowSynchronizer</xsl:variable>
	<xsl:variable name="staging_rsync_class">com.invoqsystems.apex.datasynchronizer.processor.StagingRowSynchronizerImpl</xsl:variable>
	<xsl:variable name="staging_tm_rsync_interface">com.invoqsystems.apex.datasynchronizer.processor.StagingTeamMembershipRowSynchronizer</xsl:variable>
  <xsl:variable name="staging_schedule_rsync_class">com.invoqsystems.apex.datasynchronizer.processor.StagingScheduleRowSynchronizerImpl</xsl:variable>
  <xsl:variable name="staging_holiday_schedule_rsync_class">com.invoqsystems.apex.datasynchronizer.processor.StagingHolidayScheduleRowSynchronizerImpl</xsl:variable>
  <xsl:variable name="staging_device_timeframe_rsync_class">com.invoqsystems.apex.datasynchronizer.processor.StagingDeviceTimeframeRowSynchronizerImpl</xsl:variable>
	<xsl:variable name="staging_tm_rsync_class">com.invoqsystems.apex.datasynchronizer.processor.StagingTeamMembershipRowSynchronizerImpl</xsl:variable>
	<xsl:variable name="staging_cv_rsync_interface">com.invoqsystems.apex.datasynchronizer.processor.StagingCustomValueRowSynchronizer</xsl:variable>
	<xsl:variable name="staging_cv_rsync_class">com.invoqsystems.apex.datasynchronizer.processor.StagingCustomValueRowSynchronizerImpl</xsl:variable>
	<xsl:variable name="enterprise_rsync_class">com.invoqsystems.apex.datasynchronizer.processor.EnterpriseRowSynchronizerImpl</xsl:variable>
  <xsl:variable name="ent_tm_membership_tsync_class">com.invoqsystems.apex.datasynchronizer.processor.EnterpriseTeamMembershipTableSynchronizer</xsl:variable>
  <xsl:variable name="joined_table_synchronizer_class">com.invoqsystems.apex.datasynchronizer.processor.EnterpriseJoinedTableSynchronizerImpl</xsl:variable>
  <xsl:variable name="joined_table_row_synchronizer_interface">com.invoqsystems.apex.datasynchronizer.processor.EnterpriseJoinedTableRowSynchronizer</xsl:variable>
  <xsl:variable name="joined_table_row_synchronizer_class">com.invoqsystems.apex.datasynchronizer.processor.EnterpriseJoinedTableRowSynchronizerImpl</xsl:variable>
  <xsl:variable name="password_synchronizer_class">com.invoqsystems.apex.datasynchronizer.processor.field.EnterprisePasswordFieldSynchronizerImpl</xsl:variable>

	<xsl:variable name="target">
		<xsl:value-of select="//@target" />
	</xsl:variable>
	<xsl:variable name="failure_limit">
		<xsl:value-of select="//@failure_limit" />
	</xsl:variable>

	<!-- possible actions -->
	<xsl:variable name="actionSynchronize">synchronize</xsl:variable>

	<!-- possible targets -->
	<xsl:variable name="targetAlarmpoint">alarmpoint</xsl:variable>
	<xsl:variable name="targetStaging">staging</xsl:variable>

	<!-- possible controls -->
	<xsl:variable name="required">required</xsl:variable>
	<xsl:variable name="optional">optional</xsl:variable>
	<xsl:variable name="not_present">not-present</xsl:variable>
	<xsl:variable name="new_required">new-required</xsl:variable>
	<xsl:variable name="new_optional">new-optional</xsl:variable>

	<!-- staging field strategies -->
	<xsl:variable name="requiredStrategy">RequiredStagingFieldStrategy</xsl:variable>
	<xsl:variable name="optionalStrategy">OptionalStagingFieldStrategy</xsl:variable>
	<xsl:variable name="notPresentStrategy">NotPresentStagingFieldStrategy</xsl:variable>
	<xsl:variable name="newRequiredStrategy">NewRequiredStagingFieldStrategy</xsl:variable>
	<xsl:variable name="newOptionalStrategy">NewOptionalStagingFieldStrategy</xsl:variable>

	<!-- regular versus list field synchronizer -->
	<xsl:variable name="regularFS">EnterpriseFieldSynchronizerImpl</xsl:variable>
	<xsl:variable name="listFS">EnterpriseListFieldSynchronizerImpl</xsl:variable>

	<!-- TypeMappers -->
	<xsl:variable name="stringTypeMapper">ModelStringTypeMapper</xsl:variable>
	<xsl:variable name="statusTypeMapper">ModelStatusTypeMapper</xsl:variable>
	<xsl:variable name="booleanTypeMapper">ModelBooleanTypeMapper</xsl:variable>
	<xsl:variable name="countryCodeTypeMapper">ModelCountryCodeTypeMapper</xsl:variable>
	<xsl:variable name="integerTypeMapper">ModelIntegerTypeMapper</xsl:variable>
	<xsl:variable name="integerObjectTypeMapper">ModelIntegerObjectTypeMapper</xsl:variable>
	<xsl:variable name="hashSetTypeMapper">ModelHashSetTypeMapper</xsl:variable>
	<xsl:variable name="referenceKeyTypeMapper">ModelReferenceKeyTypeMapperImpl</xsl:variable>
	<xsl:variable name="webLoginTypeTypeMapper">ModelWebLoginTypeTypeMapperImpl</xsl:variable>
	<xsl:variable name="deviceNameTypeMapper">ModelDeviceNameTypeMapper</xsl:variable>
	<xsl:variable name="deviceStatusTypeMapper">ModelDeviceStatusTypeMapper</xsl:variable>
	<xsl:variable name="coverageStatusTypeMapper">ModelCoverageStatusTypeMapper</xsl:variable>
	<xsl:variable name="listTypeMapper">ModelListTypeMapper</xsl:variable>
	<xsl:variable name="customTypeTypeMapper">ModelCustomTypeTypeMapper</xsl:variable>
	<xsl:variable name="predicateListValueTypeMapper">ModelPredicateListValueTypeMapper</xsl:variable>
	<xsl:variable name="scheduleFrequencyTypeMapper">ModelScheduleFrequencyTypeMapper</xsl:variable>
	<xsl:variable name="isRoundRobinTypeMapper">ModelIsRoundRobinTypeMapper</xsl:variable>

	<!-- Field names -->
	<xsl:variable name="external_key_field">external_key</xsl:variable>
	<xsl:variable name="status_field">status</xsl:variable>

	<!-- Row Synchronizers -->
  <xsl:variable name="owned_object_row_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.EnterpriseOwnedObjectRowSynchronizerImpl</xsl:variable>

  <!-- Field Synchronizers -->
  <xsl:variable name="enterprise_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.EnterpriseFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_list_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.EnterpriseListFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_24x7_coverage_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.Enterprise24x7CoverageFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_web_login_type_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.EnterpriseWebLoginTypeFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_address_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.EnterpriseAddressFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_attribute_category_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.EnterpriseAttributeCategoryFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_joined_table_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.EnterpriseJoinedTableFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_custom_value_user_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.EnterpriseCustomValueUserFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_team_owner_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.EnterpriseTeamOwnerFieldSynchronizerImpl</xsl:variable>
  <xsl:variable name="enterprise_predicate_event_domain_field_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.field.EnterprisePredicateEventDomainFieldSynchronizerImpl</xsl:variable>

  <!-- Validators -->
  <xsl:variable name="non_null_field_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.NonNullFieldValidator</xsl:variable>
  <xsl:variable name="non_empty_set_field_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.NonEmptySetFieldValidator</xsl:variable>
  <xsl:variable name="unique_sitename_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.UniqueSiteNameValidator</xsl:variable>
  <xsl:variable name="predicate_pc_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.PredicatePreCommitValidator</xsl:variable>
  <xsl:variable name="default_list_object_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.DefaultListObjectValidatorImpl</xsl:variable>
  <xsl:variable name="non_null_list_object_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.NonNullListObjectValidatorImpl</xsl:variable>
  <xsl:variable name="supervisor_list_object_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.SupervisorListObjectValidatorImpl</xsl:variable>
  <xsl:variable name="person_supervisor_list_object_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.PersonSupervisorListObjectValidatorImpl</xsl:variable>
  <xsl:variable name="custom_field_type_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.CustomFieldTypeValidator</xsl:variable>
  <xsl:variable name="custom_field_name_validator">com.invoqsystems.apex.datasynchronizer.processor.field.validator.CustomFieldPreCommitValidator</xsl:variable>

	<!-- Sync Direction -->
	<xsl:variable name="to_staging">S</xsl:variable>
	<xsl:variable name="to_alarmpoint">A</xsl:variable>

	<xsl:variable name="direction">
		<xsl:choose>
			<xsl:when	test="translate($target,$uc,$lc) = translate($targetAlarmpoint,$uc,$lc)">
				<xsl:value-of select="$to_alarmpoint"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$to_staging"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

  <!-- Data source to use, by spring bean name -->
  <xsl:variable name="alarmPointDataSource">alarmPointDataSource</xsl:variable>

	<!-- Company Synchronizer Implementations -->
	<!-- Four cases
		1-> Alarmpoint Synchronize :: EnterpriseSynchronizerImpl
		2-> Staging Synchronize :: StagingSynchronizerImpl
	-->
	<xsl:variable name="companySynchronizerType">
		<xsl:choose>
			<xsl:when	test="$direction=$to_alarmpoint">
				<xsl:text>EnterpriseCompanySynchronizerImpl</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>StagingCompanySynchronizerImpl</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<!-- AbstractSynchronizer -->
	<xsl:template match="alarmpoint-synchronizer">
		<beans default-lazy-init="true">
			<import
	      resource="classpath:resources/spring/integration/auditReportContext.xml" />
			<import
	      resource="classpath:resources/spring/integration/transactionalServiceContext.xml" />
			<import
				resource="classpath:resources/spring/dataApplicationContext.xml" />
			<import
				resource="classpath:resources/spring/filteredServiceContext.xml" />
			<import
				resource="classpath:resources/spring/integration/licenseApplicationContext.xml" />
			<import
	      resource="classpath:resources/spring/integration/connectionContext.xml" />
			<import
	      resource="classpath:resources/spring/integration/quartzManagementContext.xml" />

      <!-- Declares Singleton Validators -->
      <bean name="nonNullFieldValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$non_null_field_validator"/>
        </xsl:attribute>        
      </bean>

      <bean name="nonEmptySetFieldValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$non_empty_set_field_validator"/>
        </xsl:attribute>        
      </bean>

      <bean name="uniqueSiteNameValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$unique_sitename_validator"/>
        </xsl:attribute>
        <property name="siteService" ref="filteredSiteService" />
      </bean>

      <bean name="customFieldTypeValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$custom_field_type_validator"/>
        </xsl:attribute>
        <property name="customService" ref="filteredCustomService" />
      </bean>

      <bean name="predicatePreCommitValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$predicate_pc_validator"/>
        </xsl:attribute>
        <property name="predicateService" ref="dataSyncEventDomainPredicateService" />
        <property name="ownerField">
          <value><xsl:value-of select="$event_domain_key_field"/></value>
        </property>
        <property name="nameField">
          <value><xsl:value-of select="$predicate_name_field"/></value>
        </property>
      </bean>

      <bean name="customFieldPreCommitValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$custom_field_name_validator"/>
        </xsl:attribute>
        <property name="customService" ref="dataSyncCustomFieldService" />
        <property name="nameField">
          <value><xsl:value-of select="$custom_field_name_field"/></value>
        </property>
      </bean>

      <bean name="defaultListObjectValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$default_list_object_validator"/>
        </xsl:attribute>
      </bean>
      <bean name="nonNullListObjectValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$non_null_list_object_validator"/>
        </xsl:attribute>
      </bean>
      <bean name="supervisorListObjectValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$supervisor_list_object_validator"/>
        </xsl:attribute>
      </bean>
      <bean name="personSupervisorListObjectValidator">
        <xsl:attribute name="class">
          <xsl:value-of select="$person_supervisor_list_object_validator"/>
        </xsl:attribute>
        <property name="syncPersonService">
          <ref bean="dataSyncPersonService" />
        </property>
      </bean>

			<!-- Staging Credentials and Reporting for Each Company -->
			<xsl:for-each select="company">
				<xsl:if
					test="not( translate(@one_db,$uc,$lc) = translate('true',$uc,$lc) )">
					<bean>
						<xsl:attribute name="id">
							<xsl:value-of select="translate(@name, ' ', '')" />Staging</xsl:attribute>
						<xsl:attribute name="class">com.invoqsystems.apex.dao.AlarmPointDataSource</xsl:attribute>
						<xsl:attribute name="init-method">initialize</xsl:attribute>
            <property name="connectionInfo">
              <bean name="dbConnection" class="com.invoqsystems.apex.dao.DatabaseConnectionImpl" init-method="initialize">
                <property name="dbPropertyFile">
                  <value>
                    <xsl:value-of
                      select="staging_area/@properties_file" />
                  </value>
                </property>
                <property name="dbConfigurationCipher">
                  <ref bean="dbConfigurationCipher" />
                </property>
              </bean>
            </property>
						<property name="comboPooledDataSource">
							<ref> 
								<xsl:attribute name="bean" ><xsl:value-of select="translate(@name, ' ', '')"/>ComboPooledDataSource</xsl:attribute>
							</ref>
						</property>
					</bean>
					
					<bean>
						<xsl:attribute name="id"><xsl:value-of select="translate(@name, ' ', '')"/>ComboPooledDataSource</xsl:attribute>
						<xsl:attribute name="class">com.mchange.v2.c3p0.ComboPooledDataSource</xsl:attribute>
						<property name="acquireIncrement" value="1" />
						<property name="checkoutTimeout" value="0" />
						<property name="idleConnectionTestPeriod"
							value="10000" />
						<property name="minPoolSize" value="5" />
						<property name="maxPoolSize" value="100" />
						<property name="maxStatements" value="3" />
					</bean>
				</xsl:if>

				<bean>
					<xsl:attribute name="id">
						<xsl:value-of select="translate(@name, ' ', '')" />ReportKeeper</xsl:attribute>
					<xsl:attribute name="class">com.invoqsystems.apex.datasynchronizer.reporting.ReportKeeperImpl</xsl:attribute>
					<property name="companyName">
						<value>
							<xsl:value-of select="@name" />
						</value>
					</property>
				</bean>
				
				<bean>
					<xsl:attribute name="id">
						<xsl:value-of select="translate(@name, ' ', '')" />RowReporter</xsl:attribute>
					<xsl:attribute name="class">com.invoqsystems.apex.datasynchronizer.processor.RowReporter</xsl:attribute>
          <xsl:call-template name="include_audit_service" />
					<property name="reportKeeper">
						<ref>
							<xsl:attribute name="bean">
								<xsl:value-of select="translate(@name, ' ', '')" />
								<xsl:text>ReportKeeper</xsl:text>
							</xsl:attribute>
						</ref>
					</property>
				</bean>

				<bean>
					<xsl:attribute name="id">
						<xsl:value-of select="translate(@name, ' ', '')" />JoinedTableRowReporter</xsl:attribute>
					<xsl:attribute name="class">com.invoqsystems.apex.datasynchronizer.processor.JoinedTableRowReporter</xsl:attribute>
          <xsl:call-template name="include_audit_service" />
					<property name="reportKeeper">
						<ref>
							<xsl:attribute name="bean">
								<xsl:value-of select="translate(@name, ' ', '')" />
								<xsl:text>ReportKeeper</xsl:text>
							</xsl:attribute>
						</ref>
					</property>
				</bean>

			</xsl:for-each>

			<bean>
				<xsl:attribute name="id">synchronizerImpl</xsl:attribute>
				<xsl:attribute name="class">com.invoqsystems.apex.datasynchronizer.processor.SynchronizerImpl</xsl:attribute>
				<property name="reportKeepers">
					<set>
						<xsl:call-template name="reportKeepers" />
					</set>
				</property>
				<property name="companySynchronizers">
					<map>
						<xsl:call-template name="companies" />
					</map>
				</property>
        <xsl:call-template name="include_audit_service" />
        <!-- This property is taken care of programatically in the main program.
             We need both the initial and this stuff to use the same logger.
             So it is set from the log context.
             This has the side effect of not closing it before we're done.
				<property name="dataSynchronizerLogManager" >
					<ref bean="dataSynchronizerLogManager" />
				</property>
        -->
			</bean>

			<!--  Automatic variable replacements -->			
			<bean id="alarmPointPropertyConfigurer" 
    			class="org.springframework.beans.factory.config.PropertyPlaceholderConfigurer">
    			<property name="locations">
      				<list>
        				<value>resources/spring/integration/hibernate.properties</value>
        				<value>resources/spring/integration/c3p0.properties</value>
        				<value>resources/spring/integration/quartz.properties</value>
      				</list>
    			</property>
          <property name="properties">
            <props>
              <prop key="partition.use_prune">true</prop>
            </props>
          </property>
  			</bean>
			
			<!-- The row Synchronizer AOP method interceptors -->
			<bean id="rowStagingTableMarker"
			      class="com.invoqsystems.apex.datasynchronizer.processor.RowStagingTableMarker">
			</bean>

			<bean id="rowStagingTableTeamMemberMarker"
			      class="com.invoqsystems.apex.datasynchronizer.processor.RowStagingTableTeamMemberMarker">
			</bean>

			<bean id="rowStagingTableUserSupervisorMarker"
			      class="com.invoqsystems.apex.datasynchronizer.processor.RowStagingTableUserSupervisorMarker">
			</bean>

			<!-- Now include the data synchronizer services -->
			<xsl:call-template name="includeDataSyncServices" />
			
		</beans>
	</xsl:template>

	<!-- report keepers -->
	<xsl:template name="reportKeepers">
		<xsl:for-each select="company">
			<ref>
				<xsl:attribute name="bean">
					<xsl:value-of select="translate(@name,' ','')" />
					<xsl:text>ReportKeeper</xsl:text>
				</xsl:attribute>
			</ref>
		</xsl:for-each>
	</xsl:template>

	<!-- Companies -->
	<xsl:template name="companies">
		<xsl:for-each select="company">
			<entry>
				<xsl:attribute name="key">
					<xsl:value-of select="@name" />
				</xsl:attribute>
				<bean>
					<xsl:attribute name="class">
						<xsl:text>com.invoqsystems.apex.datasynchronizer.processor.</xsl:text>
						<xsl:value-of select="$companySynchronizerType" />
					</xsl:attribute>
					<property name="stagingDataSource">
						<ref>
							<xsl:attribute name="bean">
								<xsl:choose>
                  <xsl:when test="@one_db">
                    <xsl:choose>
    									<xsl:when test="translate(@one_db,$uc,$lc) = translate('true',$uc,$lc)">
    										<xsl:value-of select="$alarmPointDataSource"/>
    									</xsl:when>
                      <xsl:otherwise>
    										<xsl:value-of	select="translate(@name, ' ', '')" />
    										<xsl:text>Staging</xsl:text>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:when>
									<xsl:otherwise>
                    <xsl:value-of select="$alarmPointDataSource"/>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:attribute>
						</ref>
					</property>
          <property name="syncUser">
            <value>
              <xsl:value-of select="@user"/>
            </value>
          </property>
          <property name="alarmpointDataSource">
            <ref bean="alarmPointDataSource"/>
          </property>
					<property name="companyName">
						<value>
							<xsl:value-of select="@name" />
						</value>
					</property>
          <property name="oneDB">
            <value>
              <xsl:choose>
                <xsl:when test="@one_db">
                  <xsl:value-of select="@one_db"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>true</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </value>
          </property>
					<property name="failureLimit">
						<value>
							<xsl:value-of select="$failure_limit" />
						</value>
					</property>
					<property name="companyService">
						<ref bean="organizationService" />
					</property>
					<property name="personService">
						<ref bean="dataSyncPersonService" />
					</property>
					<property name="auditSyncService">
						<ref bean="auditSyncService" />
					</property>
					<property name="reportKeeper">
						<ref>
							<xsl:attribute name="bean">
								<xsl:value-of
									select="translate(@name, ' ', '')" />
								<xsl:text>ReportKeeper</xsl:text>
							</xsl:attribute>
						</ref>
					</property>
					<xsl:call-template name="tables" />
				</bean>
			</entry>
		</xsl:for-each>
	</xsl:template>

	<!-- Table Synchronizers -->
	<xsl:template name="tables">
		<property name="tableSynchronizers">
			<list>
				<xsl:apply-templates />
			</list>
		</property>
	</xsl:template>

	<!-- Staging Credentials -->
	<xsl:template name="inject_staging_datasource">
		<property name="stagingDataSource">
			<ref>
				<xsl:attribute name="bean">
					<xsl:choose>
            <!-- Look for company in ancestor and check if one_db is set to true -->
            <xsl:when test="ancestor::company/@one_db">
              <xsl:choose>
								<xsl:when test="translate(ancestor::company/@one_db,$uc,$lc) = translate('true',$uc,$lc)">
									<xsl:value-of select="$alarmPointDataSource"/>
								</xsl:when>
                <xsl:otherwise>
									<xsl:value-of	select="translate(ancestor::company/@name, ' ', '')" />
									<xsl:text>Staging</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
						<xsl:otherwise>
              <xsl:value-of select="$alarmPointDataSource"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
			</ref>
		</property>
	</xsl:template>

  <!-- Choose Table Synchronizer Class -->
  <xsl:template name="choose_table_synchronizer_class">
    <xsl:param name="override"/>
    <xsl:choose>
      <!-- Use override -->
      <xsl:when test="$override">
        <xsl:value-of select="$override"/>
			</xsl:when>
      <!-- Choose by target -->
      <xsl:when test="$direction=$to_alarmpoint">
        <xsl:value-of select="$enterprise_tsync_class"/>
			</xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$staging_tsync_class"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Include Audit Service for Logging -->
  <xsl:template name="include_audit_service">
    <property name="auditSyncService">
      <ref bean="auditSyncService" />
    </property>
  </xsl:template>

  <!-- Choose interface to use for row synchronizer -->
	<xsl:template name="choose_row_interface">
		<xsl:param name="sync_class"/>
		<xsl:choose>
      <!-- Choices when target is Alarmpoint -->
			<xsl:when test="$direction=$to_alarmpoint">
				<xsl:choose>
					<xsl:when	test="$sync_class=$team_membership_row_synchronizer">
						<xsl:text>com.invoqsystems.apex.datasynchronizer.processor.EnterpriseTeamMembershipTeamSynch</xsl:text>              
					</xsl:when>
					<xsl:when test="$sync_class=$owned_object_row_synchronizer">
						<xsl:text>com.invoqsystems.apex.datasynchronizer.processor.EnterpriseOwnedObjectRowSynchronizer</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>com.invoqsystems.apex.datasynchronizer.processor.EnterpriseRowSynchronizer</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
      <!-- Choices when target is Staging .. ie the only other valid target -->
      <xsl:otherwise>
				<xsl:choose>
					<xsl:when	test="$sync_class=$team_membership_row_synchronizer">
						<xsl:value-of select="$staging_tm_rsync_interface"/>
					</xsl:when>
					<xsl:when test="translate(name(),$lc,$uc) = translate('user_custom_value',$lc,$uc)">
						<xsl:value-of select="$staging_cv_rsync_interface"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$staging_rsync_interface"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Choose class to use for row synchronizer -->
	<xsl:template name="choose_row_class">
    <xsl:param name="sync_class"/>
    <xsl:variable name="nodeName"><xsl:value-of select="translate(name(),$lc,$uc)"/></xsl:variable>
		<xsl:choose>
			<!-- Choices when target is Alarmpoint -->
			<xsl:when test="$direction=$to_alarmpoint">
				<xsl:choose>
					<xsl:when test="$sync_class">
						<xsl:value-of select="$sync_class" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$enterprise_rsync_class"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<!-- Choices when target is Stagining -->
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when	test="$sync_class=$team_membership_row_synchronizer">
						<xsl:value-of select="$staging_tm_rsync_class"/>
					</xsl:when>
					<xsl:when test="$nodeName = translate('user_custom_value',$lc,$uc)">
						<xsl:value-of select="$staging_cv_rsync_class"/>
					</xsl:when>
          <xsl:when test="$nodeName = translate('recurrent_coverages',$lc,$uc)">
						<xsl:value-of select="$staging_schedule_rsync_class"/>
          </xsl:when>
          <xsl:when test="$nodeName = translate('holiday_coverages',$lc,$uc)">
            <xsl:value-of select="$staging_holiday_schedule_rsync_class"/>
          </xsl:when>
          <xsl:when test="$nodeName = translate('device_timeframe',$lc,$uc)">
						<xsl:value-of select="$staging_device_timeframe_rsync_class"/>
          </xsl:when>
          <xsl:when test="$sync_class=$person_supervisor_row_synchronizer">
						<xsl:value-of select="$staging_person_supervisor_rsync"/>
          </xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$staging_rsync_class"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="row_synchronizer">
		<xsl:param name="data_synchronization_service" />
		<xsl:param name="default_table" />
		<xsl:param name="sync_class" />
		<xsl:param name="row_marker" />
		<xsl:param name="owner_field" />
    <xsl:param name="pre_commit_validator" />
		<property name="rowSynchronizer">
			<bean class="org.springframework.aop.framework.ProxyFactoryBean">
				<property name="proxyInterfaces">
					<value>
						<xsl:call-template name="choose_row_interface">
							<xsl:with-param name="sync_class" select="$sync_class"/>
						</xsl:call-template>
					</value>
				</property>
				<property name="target">
					<bean class="org.springframework.transaction.interceptor.TransactionProxyFactoryBean"
		       			  abstract="true">
						<property name="transactionManager">
							<ref bean="transactionManager"/>
						</property>
						<property name="transactionAttributes">
							<props>
								<prop key="processRow">PROPAGATION_REQUIRED</prop>
								<prop key="processModelObject">PROPAGATION_REQUIRED</prop>
							</props>
						</property>
						<property name="proxyInterfaces">
							<value>
								<xsl:call-template name="choose_row_interface">
									<xsl:with-param name="sync_class" select="$sync_class"/>
								</xsl:call-template>
							</value>
						</property>
						<property name="target">
							<bean>
								<xsl:attribute name="class">
									<xsl:call-template name="choose_row_class">
										<xsl:with-param name="sync_class" select="$sync_class"/>
									</xsl:call-template>
								</xsl:attribute>
								<property name="dataSynchronizationService">
									<ref >
										<xsl:attribute name="bean">
											<xsl:value-of select="$data_synchronization_service" />
										</xsl:attribute>
									</ref>
								</property>
								<xsl:call-template name="include_audit_service" />
								<xsl:if test="$sync_class=$team_membership_row_synchronizer and 
															$direction=$to_alarmpoint">
									<property name="teamMembershipService">
										<ref bean="dataSyncTeamMembershipService" />
									</property>
								</xsl:if>
								<xsl:if test="$sync_class=$owned_object_row_synchronizer and 
															$direction=$to_alarmpoint">
									<property name="ownerField">
                    <value>
                      <xsl:value-of select="$owner_field"/>
										</value>
									</property>
								</xsl:if>
								<property name="tableName">
									<value>
                  <xsl:call-template name="choose_row_staging_table_name">
                    <xsl:with-param name="default_table" select="$default_table"/>
									</xsl:call-template>
									</value>
								</property>
								<xsl:call-template name="inject_staging_datasource" />
								<xsl:call-template name="row_report_keeper" />
								<!-- Field Synchronizers -->
								<property name="fieldSynchronizers">
									<map>
										<xsl:apply-templates/>
									</map>
								</property>
                <xsl:if test="$pre_commit_validator">
                  <property name="preCommitRowValidator">
                    <ref>
                      <xsl:attribute name="bean"><xsl:value-of select="$pre_commit_validator"/></xsl:attribute>
                    </ref>
                  </property>
                </xsl:if>
							</bean>
						</property>
						<property name="preInterceptors">
							<list>
								<ref bean="hibernateInterceptor" />
							</list>
						</property>
					</bean>
				</property>
				<property name="interceptorNames">
					<list>
						<xsl:call-template name="inject_row_interceptors">
							<xsl:with-param name="row_marker" select="$row_marker"/>
						</xsl:call-template>
					</list>
    			</property>
			</bean>
		</property>
	</xsl:template>

  <!-- Choose staging table name for row_synchronizer -->
  <xsl:template name="choose_row_staging_table_name">
    <xsl:param name="default_table"/>
		<xsl:choose>
			<xsl:when test="@stage_table">
				<xsl:value-of select="@stage_table" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$default_table" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="row_report_keeper">
		<property name="reportKeeper">
			<ref>
				<xsl:attribute name="bean">
					<xsl:value-of select="translate(ancestor::company/@name, ' ', '')" />
					<xsl:text>ReportKeeper</xsl:text>
				</xsl:attribute>
			</ref>
		</property>
	</xsl:template>

  <!-- Interceptors for row synchronizer -->
	<xsl:template name="inject_row_interceptors">
		<xsl:param name="row_marker"/>
		<value>
			<xsl:choose>
				<xsl:when test="$row_marker">
					<xsl:value-of select="$row_marker" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>rowStagingTableMarker</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</value>
	<!-- Do Not use the standard row reporter for team memberships -->
    <!-- because the class has a inner processRow for the actual members -->
    <xsl:if test="$row_marker!='rowStagingTableTeamMemberMarker'">
  		<value>
  			<xsl:value-of select="translate(ancestor::company/@name, ' ', '')" />
  			<xsl:text>RowReporter</xsl:text>
  		</value>
    </xsl:if>
	</xsl:template>

	<!-- field properties -->
	<xsl:template name="field_class">
		<xsl:choose>
			<xsl:when
				test="translate(name(current()),$uc,$lc)=translate($group_key_field,$uc,$lc)">
				<xsl:attribute name="class">
          <xsl:value-of select="$enterprise_team_owner_field_synchronizer"/>
				</xsl:attribute>
				<property name="teamService">
					<ref bean="teamService" />
				</property>
			</xsl:when>
      <xsl:when
        test="translate(name(current()),$uc,$lc)=translate($event_domain_key_field,$uc,$lc)">
        <xsl:attribute name="class">
          <xsl:value-of select="$enterprise_predicate_event_domain_field_synchronizer"/>
        </xsl:attribute>
				<property name="subscriptionService">
					<ref bean="subscriptionService" />
				</property>
      </xsl:when>
			<xsl:otherwise>
				<xsl:attribute name="class">
          <xsl:value-of select="$enterprise_field_synchronizer"/>
				</xsl:attribute>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="model_property_name">
		<xsl:param name="model_property_name" />
		<property name="modelPropertyName">
			<xsl:choose>
				<xsl:when test="@property_name">
					<value>
						<xsl:value-of select="@property_name" />
					</value>
				</xsl:when>
				<xsl:otherwise>
					<value>
						<xsl:value-of select="$model_property_name" />
					</value>
				</xsl:otherwise>
			</xsl:choose>
		</property>
	</xsl:template>

	<xsl:template name="stage_field_name">
	  <xsl:param name="child_external_key"/>
		<property name="stageFieldName">
			<value>
				<xsl:choose>
					<xsl:when test="@stage_name">
						<xsl:value-of select="@stage_name" />
					</xsl:when>
					<xsl:when test="$child_external_key">
						<xsl:value-of select="$child_external_key" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="name()" />
					</xsl:otherwise>
				</xsl:choose>
			</value>
		</property>
	</xsl:template>

	<xsl:template name="staging_field_strategy">
		<property name="stagingFieldStrategy">
			<bean>
				<xsl:attribute name="class">
					<xsl:text>com.invoqsystems.apex.datasynchronizer.processor.field.</xsl:text>
					<xsl:call-template name="selectStrategy" />
				</xsl:attribute>
        <xsl:call-template name="include_audit_service" />
			</bean>
		</property>
	</xsl:template>

	<!-- select defaulting -->
	<!-- decides which implementation of fieldSynchronizer to use -->
	<xsl:template name="fieldDefaulter">
		<xsl:choose>
			<xsl:when test="@default">
				<property name="fieldDefaulter">
					<bean>
						<xsl:attribute name="class">com.invoqsystems.apex.datasynchronizer.processor.field.FieldDefaulterImpl</xsl:attribute>
						<property name="defaultValue">
							<value><xsl:value-of select="@default"/></value>
						</property>
					</bean>
				</property>			
			</xsl:when>
			<xsl:otherwise>
				<property name="fieldDefaulter">
					<bean>
						<xsl:attribute name="class">com.invoqsystems.apex.datasynchronizer.processor.field.FieldNoDefaulterImpl</xsl:attribute>
					</bean>
				</property>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- select staging field strategy -->
	<xsl:template name="selectStrategy">
    <xsl:variable name="overwrite"><xsl:value-of select="translate(@overwrite,$uc,$lc) = 'true'"/></xsl:variable>
		<xsl:choose>
      <xsl:when test="not(@overwrite)">
        <xsl:choose>
    			<xsl:when
    				test="translate(@control,$uc,$lc) = translate($required,$uc,$lc)">
    				<xsl:value-of select="$requiredStrategy" />
    			</xsl:when>
    			<xsl:when
    				test="translate(@control,$uc,$lc) = translate($optional,$uc,$lc)">
    				<xsl:value-of select="$optionalStrategy" />
    			</xsl:when>
     			<xsl:when
     				test="translate(@control,$uc,$lc) = translate($not_present,$uc,$lc)">
     				<xsl:value-of select="$notPresentStrategy" />
     			</xsl:when>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
    			<xsl:when
    				test="translate(@control,$uc,$lc) = translate($required,$uc,$lc) and $overwrite='true'">
    				<xsl:value-of select="$requiredStrategy" />
    			</xsl:when>
    			<xsl:when
    				test="translate(@control,$uc,$lc) = translate($optional,$uc,$lc) and $overwrite='true'">
    				<xsl:value-of select="$optionalStrategy" />
    			</xsl:when>
    			<xsl:when
    				test="translate(@control,$uc,$lc) = translate($required,$uc,$lc) and $overwrite='false'">
    				<xsl:value-of select="$newRequiredStrategy" />
    			</xsl:when>
    			<xsl:when
    				test="translate(@control,$uc,$lc) = translate($optional,$uc,$lc) and $overwrite='false'">
    				<xsl:value-of select="$newOptionalStrategy" />
    			</xsl:when>
        </xsl:choose>
      </xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="field_model_type_mapper">
	  <xsl:param name="mapper_class"/>
	  <xsl:param name="mapper_properties"/>
		<property name="modelTypeMapper">
			<bean>
				<xsl:attribute name="class">
					<xsl:text>com.invoqsystems.apex.datasynchronizer.processor.field.</xsl:text>
					<xsl:choose> <!-- choose how to define class of mapper -->
					  <xsl:when test="$mapper_class">
					    <xsl:value-of select="$mapper_class" /> <!-- defined at field level -->
					  </xsl:when>
					  <xsl:otherwise>
                                    <xsl:call-template name="selectTypeMapperClass" /> <!-- choose class bye field type -->
					  </xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>

        <xsl:choose>
          <xsl:when test="$mapper_properties">
            <property name="dataSynchronizationService">
              <ref>
                <xsl:attribute name="bean">
                  <xsl:value-of select="$mapper_properties"/>
								</xsl:attribute>
							</ref>
						</property>
					</xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="selectTypeMapperProperties" />
					</xsl:otherwise>
				</xsl:choose>
        <xsl:call-template name="include_audit_service" />
			</bean>
		</property>
	</xsl:template>

	<!-- select the type mapper by field type  -->
	<xsl:template name="selectTypeMapperClass">
		<xsl:choose>
			<xsl:when
				test="translate(name(),$uc,$lc) = translate($status_field,$uc,$lc)">
				<xsl:choose>
					<xsl:when
						test="translate(name(current()/..),$uc,$lc) = translate('user',$uc,$lc)">
						<xsl:value-of select="$statusTypeMapper" />
					</xsl:when>
					<xsl:when
						test="translate(name(current()/..),$uc,$lc) = translate('group',$uc,$lc)">
						<xsl:value-of
							select="$coverageStatusTypeMapper" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$deviceStatusTypeMapper" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$stringTypeMapper" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Add the properties for the type mapper -->
	<xsl:template name="selectTypeMapperProperties">
		<xsl:choose>
			<xsl:when
				test="translate(name(),$uc,$lc) = translate($role_list_field,$uc,$lc)">
				<property name="dataSynchronizationService">
					<ref bean="dataSyncAccessService" />
				</property>
				<property name="listObjectValidator">
          <ref bean="nonNullListObjectValidator"/>
				</property>
				<property name="fieldName">
					<value><xsl:value-of select="name()"/></value>
				</property>
				<xsl:call-template name="field_report_keeper" />
			</xsl:when>
			<xsl:when
				test="translate(name(),$uc,$lc) = translate($supervisor_list_field,$uc,$lc)">
				<property name="dataSynchronizationService">
					<ref bean="dataSyncPersonService" />
				</property>
				<property name="listObjectValidator">
          <ref bean="supervisorListObjectValidator"/>
				</property>
				<property name="fieldName">
					<value><xsl:value-of select="name()"/></value>
				</property>
				<xsl:call-template name="field_report_keeper" />
			</xsl:when>
			<xsl:when
				test="translate(name(),$uc,$lc) = translate($person_supervisor_list_field,$uc,$lc)">
				<property name="dataSynchronizationService">
					<ref bean="dataSyncPersonService" />
				</property>
				<property name="listObjectValidator">
          <ref bean="personSupervisorListObjectValidator"/>
				</property>
				<property name="fieldName">
					<value><xsl:value-of select="name()"/></value>
				</property>
				<xsl:call-template name="field_report_keeper" />
			</xsl:when>
			<xsl:when
				test="translate(name(),$uc,$lc) = translate($observer_list_field,$uc,$lc)">
				<property name="dataSynchronizationService">
					<ref bean="dataSyncAccessService" />
				</property>
				<property name="listObjectValidator">
          <ref bean="nonNullListObjectValidator"/>
				</property>
				<property name="fieldName">
					<value><xsl:value-of select="name()"/></value>
				</property>
				<xsl:call-template name="field_report_keeper" />
			</xsl:when>
			<xsl:when
				test="translate(name(),$uc,$lc) = translate($attribute_category_key_field,$uc,$lc)">
				<property name="dataSynchronizationService">
					<ref bean="dataSyncCustomAttributeCategoryService" />
				</property>
			</xsl:when>
			<xsl:otherwise></xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="field_report_keeper">
		<property name="reportKeeper">
			<ref>
				<xsl:attribute name="bean">
					<xsl:value-of
						select="translate(ancestor::company/@name, ' ', '')" />
					<xsl:text>ReportKeeper</xsl:text>
				</xsl:attribute>
			</ref>
		</property>
	</xsl:template>
  
    <xsl:template name="special_properties">
      <xsl:if test="translate(name(),$uc,$lc) = translate($add_24x7_coverage_field,$uc,$lc)" >
        <property name="groupService" >
          <ref bean="dataSyncGroupService" />
        </property>
      </xsl:if>
      <xsl:if test="translate(name(),$uc,$lc) = translate($web_login_type_field,$uc,$lc)" >
        <property name="personService" >
          <ref bean="dataSyncPersonService" />
        </property>
      </xsl:if>
      <xsl:if test="translate(name(),$uc,$lc) = translate($web_password_field,$uc,$lc)" >
        <property name="type" >
          <value>webPassword</value>
        </property>
      </xsl:if>
      <xsl:if test="translate(name(),$uc,$lc) = translate($phone_password_field,$uc,$lc)" >
        <property name="type" >
          <value>phonePassword</value>
        </property>
      </xsl:if>
    </xsl:template>

	<!-- End field properties -->

  <!-- Handles a entry in the collection of field synchroniers -->
	<xsl:template name="call_fields">
    <xsl:param name="synchronizer_class" />
		<xsl:param name="model_property_name" />
		<xsl:param name="mapper_class" />
		<xsl:param name="child_external_key" />
    <xsl:param name="mapper_properties"/>
    <xsl:param name="validatorBean"/>
    
    <xsl:variable name="required">R</xsl:variable>
    <xsl:variable name="optional">O</xsl:variable>

    <xsl:variable name="control">
      <xsl:choose>
        <xsl:when test="translate(@control,$uc,$lc) = 'required'">
          <xsl:value-of select="$required"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$optional"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

		<entry>
			<xsl:attribute name="key">
				<xsl:value-of select="name()" />
			</xsl:attribute>
			<bean>
        <xsl:call-template name="core_field_properties">
          <xsl:with-param name="synchronizer_class" select="$synchronizer_class"/>
          <xsl:with-param name="model_property_name" select="$model_property_name"/>
          <xsl:with-param name="mapper_class" select="$mapper_class"/>
          <xsl:with-param name="child_external_key" select="$child_external_key"/>
          <xsl:with-param name="mapper_properties" select="$mapper_properties"/>
				</xsl:call-template>
        <xsl:call-template name="include_audit_service" />

        <property name="validators">
          <list>
            <xsl:if test="$control=$required">
              <ref bean="nonNullFieldValidator"/>
            </xsl:if>
            <xsl:if test="$validatorBean">
              <ref>
                <xsl:attribute name="bean">
                  <xsl:value-of select="$validatorBean"/>
                </xsl:attribute>
              </ref>
            </xsl:if>
          </list>
        </property>
			</bean>
		</entry>
	</xsl:template>

  <!-- Handles the core properties of a typical field synchronizer -->
	<xsl:template name="core_field_properties">
    <xsl:param name="synchronizer_class" />
		<xsl:param name="model_property_name" />
		<xsl:param name="mapper_class" />
		<xsl:param name="child_external_key" />
    <xsl:param name="mapper_properties"/>
 		<!-- field_class must be called first -->
    <xsl:choose>
      <!-- if class is specified then use it -->
      <xsl:when test="$synchronizer_class">
				<xsl:attribute name="class">
          <xsl:value-of select="$synchronizer_class"/>
				</xsl:attribute>
			</xsl:when>
      <!-- otherwise choose by field type -->
      <xsl:otherwise> 
        <xsl:call-template name="field_class" /> 
			</xsl:otherwise>
		</xsl:choose>
    <xsl:call-template name="fieldDefaulter" />
    <xsl:call-template name="model_property_name">
      <xsl:with-param name="model_property_name" select="$model_property_name" />
    </xsl:call-template>
    <xsl:call-template name="stage_field_name">
      <xsl:with-param name="child_external_key" select="$child_external_key"/>
    </xsl:call-template>
    <xsl:call-template name="staging_field_strategy" />
    <xsl:call-template name="field_model_type_mapper">
      <xsl:with-param name="mapper_class" select="$mapper_class" />
      <xsl:with-param name="mapper_properties" select="$mapper_properties" />
    </xsl:call-template>
		<xsl:call-template name="field_report_keeper" />
    <xsl:call-template name="special_properties" />
	</xsl:template>

	<!-- The following templates handle one type of field -->
	<!-- Note that if no mapper class is specified then a match will be done on type, the default mapper is String -->
	<xsl:template match="external_key">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">externalKey</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="status">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">status</xsl:with-param>
		</xsl:call-template>
	</xsl:template>	
	<!-- End Maping of element to ModelPropertyName -->
	
	<!-- Choose staging table name -->
      <xsl:template name="choose_staging_table_name">
        <xsl:param name="default_name" />
          <property name="stagingTableName">
            <xsl:choose>
              <xsl:when test="@stage_table">
                <value>
                  <xsl:value-of select="@stage_table" />
                </value>
              </xsl:when>
              <xsl:otherwise>
                <value>
                  <xsl:value-of select="$default_name" />
                </value>
              </xsl:otherwise>
            </xsl:choose>
          </property>
      </xsl:template>
      
      <!-- Inject ReportKeeper -->
      <xsl:template name="inject_report_keeper">
        <property name="reportKeeper">
          <ref>
            <xsl:attribute name="bean">
              <xsl:value-of
                select="translate(ancestor::company/@name, ' ', '')" />
              <xsl:text>ReportKeeper</xsl:text>
            </xsl:attribute>
          </ref>
        </property>
      </xsl:template>

</xsl:stylesheet>
