<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:template name="joined_table_synchronizer">
    <xsl:param name="joined_table_name"/>
    <xsl:param name="parent_external_key"/>
    <xsl:param name="model_property_name"/>
    <xsl:param name="child_external_key"/>
    <xsl:param name="mapper_class"/>
    <xsl:param name="mapper_properties"/>

    <xsl:attribute name="class">
      <xsl:value-of select="$joined_table_synchronizer_class"/>
    </xsl:attribute>
    <xsl:call-template name="inject_staging_datasource"/>
    <xsl:call-template name="choose_staging_table_name">
      <xsl:with-param name="default_name" select="$joined_table_name"/>
    </xsl:call-template>
    <property name="parentLinkField">
    	<value>
    	  <xsl:choose>
    	    <xsl:when test="@parent_external_key">
          <xsl:value-of select="@parent_external_key" />
    	    </xsl:when>
    	    <xsl:otherwise>
            <xsl:value-of select="$parent_external_key"/>
    	    </xsl:otherwise>
    	  </xsl:choose>
    	</value>
    </property>
    <!-- Call Joined Row Synchronizer -->
    <xsl:call-template name="joined_row_synchronizer">
      <xsl:with-param name="default_table" select="$parent_external_key"/>
      <xsl:with-param name="model_property_name" select="$model_property_name"/>
      <xsl:with-param name="mapper_class" select="$mapper_class"/>
      <xsl:with-param name="mapper_properties" select="$mapper_properties"/>
      <xsl:with-param name="child_external_key">
        <xsl:choose>
          <xsl:when test="@child_external_key">
            <xsl:value-of select="@child_external_key" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$child_external_key" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="inject_report_keeper"/>
  </xsl:template>
  
  <xsl:template name="joined_row_synchronizer">
    <xsl:param name="default_table" />
    <xsl:param name="model_property_name" />
    <xsl:param name="child_external_key"/>
    <xsl:param name="mapper_class" />
    <xsl:param name="mapper_properties"/>
    <property name="rowSynchronizer">
      <bean class="org.springframework.aop.framework.ProxyFactoryBean">
        <property name="proxyInterfaces">
          <value>
            <xsl:value-of select="$joined_table_row_synchronizer_interface"/>
          </value>
        </property>
        <property name="target">
          <bean class="org.springframework.transaction.interceptor.TransactionProxyFactoryBean" abstract="true">
               <property name="transactionManager">
                 <ref bean="transactionManager"/>
               </property>
               <property name="transactionAttributes">
                 <props>
                   <prop key="processRow">PROPAGATION_REQUIRED</prop>
                 </props>
               </property>
              <property name="proxyInterfaces">
                <value>
                  <xsl:value-of select="$joined_table_row_synchronizer_interface"/>
                </value>
              </property>
              <property name="target">
                <bean>
                  <xsl:attribute name="class">
                        <xsl:value-of select="$joined_table_row_synchronizer_class"/>
                  </xsl:attribute>
                  <property name="tableName">
                    <xsl:choose>
                      <xsl:when test="@stage_table">
                        <value>
                          <xsl:value-of select="@stage_table" />
                        </value>
                      </xsl:when>
                      <xsl:otherwise>
                        <value>
                          <xsl:value-of select="$default_table" />
                        </value>
                      </xsl:otherwise>
                    </xsl:choose>
                  </property>
                  <xsl:call-template name="inject_staging_datasource" />
                  <xsl:call-template name="row_report_keeper" />
                  <!-- Joined Table Field Synchronizers -->
                  <property name="joinedTableFieldSynchronizers">
                    <map>
                      <xsl:call-template name="call_fields">
                        <xsl:with-param name="synchronizer_class" select="$enterprise_joined_table_field_synchronizer"/>
                        <xsl:with-param name="model_property_name" select="$model_property_name"/>
                        <xsl:with-param name="child_external_key" select="$child_external_key"/>
                        <xsl:with-param name="mapper_class" select="$mapper_class"/>
                        <xsl:with-param name="mapper_properties" select="$mapper_properties"/>
                      </xsl:call-template>
                    </map>
                  </property>
                </bean>
              </property>
              <property name="preInterceptors">
                <list>
                  <ref bean="hibernateInterceptor" />
                </list>
              </property>
          </bean>
        </property>
        <property name="interceptorNames">
              <list>
                <value>
                  <xsl:value-of select="translate(ancestor::company/@name, ' ', '')" />
                  <xsl:text>JoinedTableRowReporter</xsl:text>
                </value>
              </list>
         </property>
      </bean>
    </property>
  </xsl:template>
  
</xsl:stylesheet>
